/*****************************************************************************
* Copyright 2013-2014 - Alexander 'Wareya' Nadeau <wareya@gmail.com>
* 
* This software is provided 'as-is', without any express or implied warranty. In
* no event will the authors be held liable for any damages arising from the use
* of this software.
* 
* Permission is granted to anyone to use this software and/or its source code for
* any purpose, including commercial applications, and to alter it and distribute
* it freely, subject to the following conditions:
*
*    1. The origin of the software must not be misrepresented.
*
*    2. Modified versions of the software or its source code must not
*    be misrepresented as being the original software or source code.
*
*    3. This notice may not be removed from, or altered in, any source
*    distribution of the software.
*
*        3a. This notice is not required to be included in any compiled
*        distribution of the software, unless said distribution also
*        constitutes a source distribution.
*        
*    4. In case of use or inclusion of this software in a commercial
*    product, a proper acknowledgment of this software inside the final
*    product is required.
******************************************************************************/

// Init
var createcode;
createcode = '
    if (!instance_exists(WarChatWindow))
        instance_create(0, 0, WarChatWindow);
';

object_event_add(Spectator, ev_step, ev_step_end, createcode);

// Vars
globalvar WarChatPPS;
WarChatPPS = argument1;

// Setup
globalvar WarChatWindow, FakeMenu;
WarChatWindow = object_add();
FakeMenu = object_add();

// Fake menu for catching input if we need it (while typing)
object_event_add(FakeMenu, ev_step, ev_step_normal, '');
object_event_add(FakeMenu, ev_draw, 0, '');
object_event_add(FakeMenu, ev_keypress, vk_enter, '');
object_event_add(FakeMenu, ev_keypress, vk_escape, '
    with(WarChatWindow)
    {
        keyboard_string = "";
        typing = false;
    }
    with (FakeMenu)
        instance_destroy();
');
object_set_parent(FakeMenu, InGameMenuController);

object_set_depth(WarChatWindow, -109999);

object_event_add(WarChatWindow, ev_destroy, 0, '
    ds_list_destroy(lines);
    ds_list_destroy(times);
    ds_list_destroy(teams);
');

object_event_add(WarChatWindow, ev_create, 0, '
    hosting = global.isHost;
    
    scrollback = 256;
    
    mode = 0;
    
    x = 24;
    y = -96;
    h = 40*6;
    w = 80*6;
    inputh = 20;
    lineh = 12;
    margins = 4;
    backcolor = $1C2018;
    entryback = $000000;
    blue = $FF4040;
    red  = $4040FF;
    
    lines = ds_list_create();
    times = ds_list_create();
    teams = ds_list_create(); // 0 = none; 1 = red; 2 = blue; 4/5 = server slash admin (red/blue)
    
    typing = false;
    
    ds_list_add(lines, "This server is running WarChat version 1, for GG2 v2.7.2.");
    ds_list_add(times, current_time/30);
    ds_list_add(teams, 0);
    
    ds_list_add(lines, "Press enter to start typing in ALLCHAT.");
    ds_list_add(times, current_time/30);
    ds_list_add(teams, 0);
    
    ds_list_add(lines, "Press ctrl-enter to start typing in TEAMCHAT.");
    ds_list_add(times, current_time/30);
    ds_list_add(teams, 0);
    
    ds_list_add(lines, "Ctrl+V can paste text, Ctrl+C copies the full editing text.");
    ds_list_add(times, current_time/30);
    ds_list_add(teams, 0);
    
    b = buffer_create();
');

object_event_add(WarChatWindow, ev_step, ev_step_normal, '
    if (keyboard_check_pressed(vk_enter) and !keyboard_check(vk_shift)
        and !(instance_exists(MenuController) and !instance_exists(FakeMenu)))
    {
        typing = !typing;
        
        if (typing) // if we just began typing
        {
            instance_create(0, 0, FakeMenu);
            
            if (keyboard_check(vk_control))
                mode = 102; // 102 for team message
            else
                mode = 101; // 101 for all message
        }
        else // if we just sent a message
        {
            with (FakeMenu)
                instance_destroy();
            
            if (keyboard_string == "") // blank means break it
                break;
            keyboard_string = string_copy(keyboard_string, 1, 255);
            
            write_ubyte(b, mode);
            write_ubyte(b, string_length(keyboard_string)); // len
            write_string(b, keyboard_string); // msg
            if(global.isHost)
                PluginPacketSendTo(WarChatPPS, b, global.myself);
            else
                PluginPacketSend(WarChatPPS, b, 0);
            buffer_clear(b);
        }
        keyboard_string = "";
    }
    
    if (keyboard_check(vk_control) and keyboard_check_pressed(ord("V")) and clipboard_has_text())
    {
        keyboard_string += clipboard_get_text();
    }
    if (keyboard_check(vk_control) and keyboard_check_pressed(ord("C")) and clipboard_has_text())
    {
        clipboard_set_text(keyboard_string);
    }
    
    while (PluginPacketGetBuffer(WarChatPPS) != -1)
    {
        d = PluginPacketGetBuffer(WarChatPPS);
        
        cmd = read_ubyte(d);
        
        switch(cmd)
        {
        case 100: // announce
            len = read_ubyte(d);
            msg = read_string(d, len);
            ds_list_add(lines, msg);
            ds_list_add(times, current_time/30);
            ds_list_add(teams, 0);
            break;
            
        case 101: // all-msg
            len = read_ubyte(d);
            msg = read_string(d, len);
            
            if (global.isHost) // Serve message
            {
                msg = PluginPacketGetPlayer(WarChatPPS).name + ": " + msg;
                var buf;
                buf = buffer_create();
                
                write_ubyte(buf, 101);
                write_ubyte(buf, string_length(msg)); // len
                write_string(buf, msg); // msg
                
                with (Player)
                {
                    if (id != global.myself)
                        PluginPacketSendTo(WarChatPPS, buf, id)
                }
                buffer_destroy(buf);
            }
            
            ds_list_add(lines, msg);
            ds_list_add(times, current_time/30);
            ds_list_add(teams, 0);
            
            break;
            
        case 102: // Team message
            len = read_ubyte(d);
            msg = read_string(d, len);
            
            team = read_ubyte(d);
            
            if (global.isHost) // Serve message to teammates of sender
            {
                msg = PluginPacketGetPlayer(WarChatPPS).name + ": " + msg;
                var buf;
                buf = buffer_create();
                
                write_ubyte(buf, 102); // 102 for team message
                write_ubyte(buf, string_length(msg)); // len
                write_string(buf, msg); // msg
                
                //
                team = PluginPacketGetPlayer(WarChatPPS).team;
                if(team == TEAM_RED)
                    team = 1;
                else if(team == TEAM_BLUE)
                    team = 2;
                else
                    team = 0;
                write_ubyte(buf, team);
                //
                
                with (Player)
                {
                    if (team == PluginPacketGetPlayer(WarChatPPS).team and id != global.myself)
                        PluginPacketSendTo(WarChatPPS, buf, id)
                }
                buffer_destroy(buf);
                
                // Prevent host from "receiving" enemy messages
                if (global.myself.team != PluginPacketGetPlayer(WarChatPPS).team)
                    break;
            }
            
            ds_list_add(lines, msg);
            ds_list_add(times, current_time/30);
            ds_list_add(teams, team);
            
            break;
        }
        
        PluginPacketPop(WarChatPPS);
    }
    
    while (ds_list_size(lines) > 200)
        ds_list_delete(lines, 0);
    while (ds_list_size(times) > 200)
        ds_list_delete(times, 0);
    while (ds_list_size(teams) > 200)
        ds_list_delete(teams, 0);
');

object_event_add(WarChatWindow, ev_draw, 0, '
    if (x >= 0)
        xs = x;
    else
        xs = view_wview + x - w;
        
    if (y >= 0)
        ys = y;
    else
        ys = view_hview + y - h;
    
    xs += view_xview;
    ys += view_yview;
    
    // Draw chatbox
    if (typing)
    {
        draw_set_alpha(0.5);
        draw_set_color(backcolor);
        draw_rectangle(xs, ys, xs + w - 1, ys + h - inputh - 1, false);
        draw_set_color(entryback);
        draw_rectangle(xs, ys + h - inputh, xs + w - 1, ys + h - 1, false);
        /*draw_set_alpha(0.125);
        draw_set_color(c_black);
        draw_rectangle(xs, ys, xs + w - 1, ys + h - 1, true);*/ //Commented out because it doesnt look good
    }
    
    // Draw chat lines as appropriate
    draw_set_halign(fa_left);
    draw_set_valign(fa_top);
    draw_set_alpha(1);
    draw_set_color(c_white);
    skipline = 0;
    for (i = 0; i < ds_list_size(lines) and i < floor((h-inputh-margins*2)/lineh); i += 1)
    {
        if (!typing) // Fadeout for recent messages when not in chatbox mode
        {
            var t;
            t = current_time/30 - ds_list_find_value(times, ds_list_size(times)-i-1);
            // A case for t <= 200 is not necessary because those should always come before t > 200 messages.
            if (t > 200 and t < 400)
                draw_set_alpha((400-t)/200);
            else if (t >= 400)
                break;
        }
        var str;
        
        str = ds_list_find_value(lines, ds_list_size(lines)-i-1);
        str = string_replace_all(str, "#", "\#");
        str = string_replace_all(str, chr(10), "");
        str = string_replace_all(str, chr(13), "");
        var _str;
        _str = string_delete(str, 1, 0*59);
        str = string_delete(str, 59+1, string_length(_str)-59);
        
        // dropshadow
        draw_set_color(c_black);
        draw_text(xs+margins+1, 1+ys + h - inputh - margins - (i+1)*lineh, str);
        
        // top
        var team;
        var name;
        var namecolor;
        var isnamecolored;
        isnamecolored = 0;
        team = ds_list_find_value(teams, ds_list_size(teams)-i-1);
        if(team == 1)
            draw_set_color(red);
        else if(team == 2)
            draw_set_color(blue);
        else
            draw_set_color(c_white);
        
        // figure out name and color of name
        name = string_pos(":", str); // length
        isnamecolored = (name > 0);
        if(isnamecolored)
        {
            name = string_delete(str, name, string_length(str)-name+1); // name
            namecolor  = (ord(string_char_at(md5_bin(name), 1))/2+127);
            namecolor += (ord(string_char_at(md5_bin(name), 2))/2+127)*256;
            namecolor += (ord(string_char_at(md5_bin(name), 3))/2+127)*256*256;
        }
        
        draw_text(xs+margins, ys + h - inputh - margins - (i+1)*lineh, str);
        
        if(isnamecolored)
        {
            draw_set_color(namecolor);
            draw_text(xs+margins, ys + h - inputh - margins - (i+1)*lineh, name);
        }
    }
    if (typing)
    {
        var str;
        str = string_replace_all(keyboard_string, "#", "\#");
        str = string_replace_all(str, chr(10), "");
        str = string_replace_all(str, chr(13), "");
        var _str;
        _str = string_delete(str, 1, 0*59-string_length(global.myself.name)-2);
        str = string_delete(str, 59-string_length(global.myself.name)-2+1, string_length(_str)-59-string_length(global.myself.name)-2);
        
        // dropshadow
        draw_set_color(c_black);
        draw_text(xs+margins+1, 1+ys + h - inputh + (inputh-lineh)/2, str);
        
        // top
        if(mode == 102 and global.myself.team == TEAM_RED)
            draw_set_color(red);
        else if(mode == 102 and global.myself.team == TEAM_BLUE)
            draw_set_color(blue);
        else
            draw_set_color(c_white);
        
        draw_text(xs+margins, ys + h - inputh + (inputh-lineh)/2, str);
    }
');